@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2013 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -54,15 +54,9 @@ typedef struct touch_data touch_data;
   /* The OpenGL names for the framebuffer and renderbuffer used to
      render to this view. */
   GLuint gl_framebuffer, gl_renderbuffer;
-  UIDeviceOrientation orientation, new_orientation;
   UIView *aboutBox;
-  NSTimer *orientationTimer;
   BOOL screenLocked;
 
-  GLfloat rotation_ratio;	// ratio thru rotation anim, or -1
-  NSSize rot_from, rot_to;	// start size rect, end size rect
-  GLfloat angle_from, angle_to;	// start angle, end angle
-
 # endif /* !USE_IPHONE */
 
   GLuint textures[2];
@@ -97,11 +91,7 @@ typedef struct touch_data touch_data;
 - (IBAction) aboutClick: (id)sender;
 
 #ifdef USE_IPHONE
-- (void)setFrameSize:(NSSize)newSize;
-- (void)didRotate:(NSNotification *)notification;
-- (void)showBlurb:(NSString *)text;
 - (void)setScreenLocked:(BOOL)locked;
-- (void)initGestures;
 #endif /* USE_IPHONE */
 
 
@@ -122,7 +112,6 @@ typedef struct touch_data touch_data;
      initialBackgroundColor
 */
 
-
 #ifdef USE_IPHONE
 @property (nonatomic) int hourStyle;
 @property (nonatomic) int timeStyle;
