@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991-2014 Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991-2018 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -221,6 +221,73 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 }
 
 
+#ifdef USE_IPHONE
+- (void)initFramebuffer
+{
+
+  if (gl_framebuffer)  glDeleteFramebuffersOES  (1, &gl_framebuffer);
+  if (gl_renderbuffer) glDeleteRenderbuffersOES (1, &gl_renderbuffer);
+
+  glGenFramebuffersOES  (1, &gl_framebuffer);
+  glBindFramebufferOES  (GL_FRAMEBUFFER_OES,  gl_framebuffer);
+
+  glGenRenderbuffersOES (1, &gl_renderbuffer);
+  glBindRenderbufferOES (GL_RENDERBUFFER_OES, gl_renderbuffer);
+
+// redundant?
+//   glRenderbufferStorageOES (GL_RENDERBUFFER_OES, GL_RGBA8_OES, w, h);
+  [ogl_ctx renderbufferStorage:GL_RENDERBUFFER_OES
+           fromDrawable:(CAEAGLLayer*)self.layer];
+
+  glFramebufferRenderbufferOES (GL_FRAMEBUFFER_OES,  GL_COLOR_ATTACHMENT0_OES,
+                                GL_RENDERBUFFER_OES, gl_renderbuffer);
+
+  int err = glCheckFramebufferStatusOES (GL_FRAMEBUFFER_OES);
+  switch (err) {
+  case GL_FRAMEBUFFER_COMPLETE_OES:
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_OES:
+    NSAssert (0, @"framebuffer incomplete attachment");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_OES:
+    NSAssert (0, @"framebuffer incomplete missing attachment");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_OES:
+    NSAssert (0, @"framebuffer incomplete dimensions");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_OES:
+    NSAssert (0, @"framebuffer incomplete formats");
+    break;
+  case GL_FRAMEBUFFER_UNSUPPORTED_OES:
+    NSAssert (0, @"framebuffer unsupported");
+    break;
+/*
+  case GL_FRAMEBUFFER_UNDEFINED:
+    NSAssert (0, @"framebuffer undefined");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER:
+    NSAssert (0, @"framebuffer incomplete draw buffer");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER:
+    NSAssert (0, @"framebuffer incomplete read buffer");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE:
+    NSAssert (0, @"framebuffer incomplete multisample");
+    break;
+  case GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS:
+    NSAssert (0, @"framebuffer incomplete layer targets");
+    break;
+ */
+  default:
+    NSAssert (0, @"framebuffer incomplete, unknown error 0x%04X", err);
+    break;
+  }
+
+  check_gl_error ("OES_init");
+}
+#endif // USE_IPHONE
+
+
 - (id)initWithFrame:(NSRect)rect ownWindow:(BOOL)own_p
 {
   self = [super initWithFrame:rect];
@@ -292,6 +359,9 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   [ctx makeCurrentContext];
   check_gl_error ("makeCurrentContext");
 
+  // Get device pixels instead of points, for Retina displays.
+  self.wantsBestResolutionOpenGLSurface = YES;
+
   // Clear frame buffer ASAP, else there are bits left over from other apps.
   // #### On 10.7, this causes GL_INVALID_FRAMEBUFFER_OPERATION.
   // In Dali Clock but not XScreenSaver?
@@ -358,66 +428,7 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 
   [EAGLContext setCurrentContext: ogl_ctx];
 
-  if (gl_framebuffer)  glDeleteFramebuffersOES  (1, &gl_framebuffer);
-  if (gl_renderbuffer) glDeleteRenderbuffersOES (1, &gl_renderbuffer);
-
-  glGenFramebuffersOES  (1, &gl_framebuffer);
-  glBindFramebufferOES  (GL_FRAMEBUFFER_OES,  gl_framebuffer);
-
-  glGenRenderbuffersOES (1, &gl_renderbuffer);
-  glBindRenderbufferOES (GL_RENDERBUFFER_OES, gl_renderbuffer);
-
-// redundant?
-//   glRenderbufferStorageOES (GL_RENDERBUFFER_OES, GL_RGBA8_OES, w, h);
-  [ogl_ctx renderbufferStorage:GL_RENDERBUFFER_OES
-           fromDrawable:(CAEAGLLayer*)self.layer];
-
-  glFramebufferRenderbufferOES (GL_FRAMEBUFFER_OES,  GL_COLOR_ATTACHMENT0_OES,
-                                GL_RENDERBUFFER_OES, gl_renderbuffer);
-
-  int err = glCheckFramebufferStatusOES (GL_FRAMEBUFFER_OES);
-  switch (err) {
-  case GL_FRAMEBUFFER_COMPLETE_OES:
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_OES:
-    NSAssert (0, @"framebuffer incomplete attachment");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_OES:
-    NSAssert (0, @"framebuffer incomplete missing attachment");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_OES:
-    NSAssert (0, @"framebuffer incomplete dimensions");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_FORMATS_OES:
-    NSAssert (0, @"framebuffer incomplete formats");
-    break;
-  case GL_FRAMEBUFFER_UNSUPPORTED_OES:
-    NSAssert (0, @"framebuffer unsupported");
-    break;
-/*
-  case GL_FRAMEBUFFER_UNDEFINED:
-    NSAssert (0, @"framebuffer undefined");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER:
-    NSAssert (0, @"framebuffer incomplete draw buffer");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER:
-    NSAssert (0, @"framebuffer incomplete read buffer");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_MULTISAMPLE:
-    NSAssert (0, @"framebuffer incomplete multisample");
-    break;
-  case GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS:
-    NSAssert (0, @"framebuffer incomplete layer targets");
-    break;
- */
-  default:
-    NSAssert (0, @"framebuffer incomplete, unknown error 0x%04X", err);
-    break;
-  }
-
-  check_gl_error ("OES_init");
-
+  [self initFramebuffer];
 # endif // USE_IPHONE
 
 # ifdef DO_DOUBLE_TEXTURE
@@ -446,9 +457,6 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
      target bitmap size shifts over a boundary).
    */
   if ([self inLiveResize]) return;
-
-# else  /* USE_IPHONE */
-  [self didRotate:nil];
 # endif /* USE_IPHONE */
 
   int ow = config.width;
@@ -464,22 +472,9 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
 # endif /* __IPHONE_4_0 */
 
   float sscale = 2.5;   // use the next-larger bitmap
-# ifndef USE_IPHONE
+
   config.width  = newSize.width  * sscale;
   config.height = newSize.height * sscale;
-# else  /* USE_IPHONE */
-  switch (orientation) {
-  case UIDeviceOrientationLandscapeLeft:
-  case UIDeviceOrientationLandscapeRight:
-    config.width  = newSize.height * sscale;
-    config.height = newSize.width  * sscale;
-    break;
-  default:
-    config.width  = newSize.width  * sscale;
-    config.height = newSize.height * sscale;
-    break;
-  }
-# endif /* USE_IPHONE */
 
   render_bitmap_size (&config, 
                       &config.width, &config.height,
@@ -499,160 +494,30 @@ static NSUserDefaultsController *staticUserDefaultsController = 0;
   render_init (&config);
 
 # ifdef USE_IPHONE
-  if (rotation_ratio < 0) {		// Not currently doing a rotation
-    switch (orientation) {
-    case UIDeviceOrientationLandscapeLeft:
-    case UIDeviceOrientationLandscapeRight:
-      rot_to.width  = newSize.height;
-      rot_to.height = newSize.width;
-      break;
-    default:
-      rot_to.width  = newSize.width;
-      rot_to.height = newSize.height;
-      break;
-    }
-    rot_from   = rot_to;
-    angle_from = angle_to;
-  }
-# endif /* USE_IPHONE */
+  // Orientation changes resize the window (swapping W and H) so we need
+  // to get a new framebuffer that matches the current shape of the window,
+  // or else the aspect ratio goes wonky.
+  if (gl_framebuffer)
+    [self initFramebuffer];
+  if (aboutBox)
+    [self aboutOff];
+
+# endif
 }
 
 
 #ifdef USE_IPHONE
 
-+ (Class) layerClass
+- (void)setFrame:(NSRect)r
 {
-    return [CAEAGLLayer class];
+  [super setFrame:r];
+  [self setFrameSize:r.size];
 }
 
 
-/* Called after the device's orientation has changed.
-
-   Note: we could include a subclass of UIViewController which
-   contains a shouldAutorotateToInterfaceOrientation method that
-   returns YES, in which case Core Animation would auto-rotate our
-   View for us in response to rotation events... but, that interacts
-   badly with the EAGLContext -- if you introduce Core Animation into
-   the path, the OpenGL pipeline probably falls back on software
-   rendering and performance goes to hell.  Also, the scaling and
-   rotation that Core Animation does interacts incorrectly with the GL
-   context anyway.
-
-   So, we have to hack the rotation animation manually, in the GL world.
-*/
-- (void)didRotate:(NSNotification *)notification
++ (Class) layerClass
 {
-  if (! [self window]) return;  // Early.
-
-  UIDeviceOrientation current = [[UIDevice currentDevice] orientation];
-
-  /* If the simulator starts up in the rotated position, sometimes
-     the UIDevice says we're in Portrait when we're not -- but it
-     turns out that the UINavigationController knows what's up!
-     So get it from there.
-     
-     NOTE!  When debugging, I thought PortraitUpsideDown was coming in as
-     Portrait, but actually what was going on was, the iPhone was upside
-     down -- but at about a 30% angle from horizontal on my desk, and that
-     was being interpreted as Portrait.  Get the angle right when debugging!
-   */
-  if (current == UIDeviceOrientationUnknown) {
-    switch ([[[self window] rootViewController] interfaceOrientation]) {
-    case UIInterfaceOrientationPortrait:
-      current = UIDeviceOrientationPortrait;
-      break;
-    case UIInterfaceOrientationPortraitUpsideDown:
-      current = UIDeviceOrientationPortraitUpsideDown;
-      break;
-    case UIInterfaceOrientationLandscapeLeft:		// It's opposite day
-      current = UIDeviceOrientationLandscapeRight;
-      break;
-    case UIInterfaceOrientationLandscapeRight:
-      current = UIDeviceOrientationLandscapeLeft;
-      break;
-    default:
-      break;
-    }
-  }
-
-  /* On the iPad (but not iPhone 3GS, or the simulator) sometimes we get
-     an orientation change event with an unknown orientation.  Those seem
-     to always be immediately followed by another orientation change with
-     a *real* orientation change, so let's try just ignoring those bogus
-     ones and hoping that the real one comes in shortly...
-   */
-  if (current == UIDeviceOrientationUnknown)
-    return;
-
-  /* iOS 7 kludge: when launching non-Portrait orientation, we get a
-     Portrait event and then the right one immediately afterward.
-     So if we're still at ratio 0, just pretend that didn't happen.
-   */
-   if (orientation == UIDeviceOrientationUnknown &&
-       current     != UIDeviceOrientationUnknown &&
-       rotation_ratio == 0) {
-     rotation_ratio = -1;
-   }
-
-  if (rotation_ratio >= 0) return;	// in the midst of rotation animation
-  if (orientation == current) return;	// no change
-
-  // When transitioning to FaceUp or FaceDown, pretend there was no change.
-  if (current == UIDeviceOrientationFaceUp ||
-      current == UIDeviceOrientationFaceDown)
-    return;
-
-  new_orientation = current;		// current animation target
-  rotation_ratio = 0;			// start animating
-
-  switch (orientation) {
-  case UIDeviceOrientationLandscapeLeft:      angle_from = 90;  break;
-  case UIDeviceOrientationLandscapeRight:     angle_from = 270; break;
-  case UIDeviceOrientationPortraitUpsideDown: angle_from = 180; break;
-  default:                                    angle_from = 0;   break;
-  }
-
-  switch (new_orientation) {
-  case UIDeviceOrientationLandscapeLeft:      angle_to = 90;  break;
-  case UIDeviceOrientationLandscapeRight:     angle_to = 270; break;
-  case UIDeviceOrientationPortraitUpsideDown: angle_to = 180; break;
-  default:                                    angle_to = 0;   break;
-  }
-
-  NSRect ff = [self frame];
-
-# if defined(__IPHONE_4_0)
-  /* Make sure we are dealing with real pixels, not "virtual" pixels. */
-  if ([self respondsToSelector:@selector(contentScaleFactor)]) {
-    double s = [self contentScaleFactor];
-    ff.size.width  *= s;
-    ff.size.height *= s;
-  }
-# endif /* __IPHONE_4_0 */
-
-  switch (orientation) {
-  case UIDeviceOrientationLandscapeRight:	// from landscape
-  case UIDeviceOrientationLandscapeLeft:
-    rot_from.width  = ff.size.height;
-    rot_from.height = ff.size.width;
-    break;
-  default:					// from portrait
-    rot_from.width  = ff.size.width;
-    rot_from.height = ff.size.height;
-    break;
-  }
-
-  switch (new_orientation) {
-  case UIDeviceOrientationLandscapeRight:	// to landscape
-  case UIDeviceOrientationLandscapeLeft:
-    rot_to.width  = ff.size.height;
-    rot_to.height = ff.size.width;
-    break;
-  default:					// to portrait
-    rot_to.width  = ff.size.width;
-    rot_to.height = ff.size.height;
-    break;
-  }
+    return [CAEAGLLayer class];
 }
 
 
@@ -877,7 +742,7 @@ FAIL:
     anim.autoreverses = NO;
     anim.fromValue    = [NSNumber numberWithFloat: 1];
     anim.toValue      = [NSNumber numberWithFloat: 0];
-    anim.delegate     = self;
+//  anim.delegate     = self;
     aboutBox.layer.opacity = 0;
     [aboutBox.layer addAnimation:anim forKey:@"animateOpacity"];
   }
@@ -961,7 +826,6 @@ FAIL:
 - (void)drawRect:(NSRect)rect
 {
   NSRect framerect = [self frame];
-  NSRect framerect2 = framerect;
   NSRect torect;
 
   /**************************************************************************
@@ -974,68 +838,24 @@ FAIL:
 
 # ifdef USE_IPHONE
 
+  double s = 1;  /* Retina scale factor */
+
 #  if defined(__IPHONE_4_0)
   /* Make sure we are dealing with real pixels, not "virtual" pixels. */
   if ([self respondsToSelector:@selector(contentScaleFactor)]) {
-    double s = [self contentScaleFactor];
+    s = [self contentScaleFactor];
     framerect.size.width  *= s;
     framerect.size.height *= s;
-    framerect2 = framerect;
   }
 #  endif /* __IPHONE_4_0 */
 
-  GLfloat angle = 0;
+# else // !USE_IPHONE
 
-  // No rotation animation if this is the first time.
-  if (orientation == UIDeviceOrientationUnknown) {
-    [self didRotate:nil];
-    orientation = new_orientation;
-    rotation_ratio = -1;
-  }
-
-  if (rotation_ratio >= 0) {	// in the midst of a rotation animation
-
-#   define CLAMP180(N) while (N < 0) N += 360; while (N > 180) N -= 360
-    GLfloat f = angle_from;
-    GLfloat t = angle_to;
-    CLAMP180(f);
-    CLAMP180(t);
-    GLfloat dist = -(t-f);
-    CLAMP180(dist);
-#   undef CLAMP180
-
-    // Intermediate angle.
-    angle = f - rotation_ratio * dist;
-
-    // Intermediate frame size.
-    framerect2.size.width = rot_from.width + 
-      rotation_ratio * (rot_to.width - rot_from.width);
-    framerect2.size.height = rot_from.height + 
-      rotation_ratio * (rot_to.height - rot_from.height);
-
-    // Tick animation.  Complete rotation in 1/6th sec.
-    rotation_ratio += 6.0 / config.max_fps;
-
-    if (rotation_ratio > 1) {	// Done animating.
-      orientation = new_orientation;
-      angle = angle_to;
-      framerect2.size = rot_to;
-      rotation_ratio = -1;
-
-      // Maybe reload the bitmaps.
-      [self setFrameSize:[self frame].size];
-
-      // Check orientation again in case we rotated again while rotating:
-      // this is a no-op if nothing has changed.
-      [self didRotate:nil];
-    }
-  } else {			// Not animating a rotation.
-    angle = angle_to;
-    framerect2.size = rot_to;
-  }
-
-# endif // USE_IPHONE
+  CGFloat s = self.window.backingScaleFactor;
+  framerect.size.width  *= s;
+  framerect.size.height *= s;
 
+# endif // !USE_IPHONE
 
   float img_aspect = (float) config.width / (float) config.height;
   float win_aspect = framerect.size.width / (float) framerect.size.height;
@@ -1043,11 +863,11 @@ FAIL:
   // Scale the image to fill the window without changing its aspect ratio.
   //
   if (win_aspect > img_aspect) {
-    torect.size.height = framerect2.size.height;
-    torect.size.width  = framerect2.size.height * img_aspect;
+    torect.size.height = framerect.size.height;
+    torect.size.width  = framerect.size.height * img_aspect;
   } else {
-    torect.size.width  = framerect2.size.width;
-    torect.size.height = framerect2.size.width / img_aspect;
+    torect.size.width  = framerect.size.width;
+    torect.size.height = framerect.size.width / img_aspect;
   }
 
 
@@ -1068,6 +888,7 @@ FAIL:
     //
     if (constrainSizes) {
       int maxh = (config.time_mode == SS ? 512 : /*256*/ 200);
+      maxh *= s;
       if (torect.size.height > maxh) {
         torect.size.height = maxh;
         torect.size.width  = maxh * img_aspect;
@@ -1238,14 +1059,6 @@ FAIL:
   GLfloat tw = (GLfloat) config.width  / config.width2;
   GLfloat th = (GLfloat) config.height / config.height2;
 
-#ifdef USE_IPHONE
-  if (angle != 0) {
-    glTranslatef (qx+qw/2, qy+qh/2, 0);
-    glRotatef (-angle, 0, 0, 1);
-    glTranslatef (-qx-qw/2, -qy-qh/2, 0);
-  }
-#endif /* USE_IPHONE */
-
   if (config.left_offset != 0)
     glTranslatef (-qw * ((GLfloat) config.left_offset / config.width),
                   0, 0);
@@ -1259,6 +1072,7 @@ FAIL:
   glEnd();
   check_gl_error ("quad");
 # else /* USE_IPHONE */
+
   GLfloat vertices[] = {
     qx,    qy,    0,
     qx+qw, qy,    0,
@@ -1321,11 +1135,7 @@ FAIL:
     render_once (&config);
     // [[self window] invalidateShadow]; // windows with shadows flicker...
 
-# ifdef USE_IPHONE
     [self drawRect:[self frame]];
-# else  /* !USE_IPHONE */
-    [self setNeedsDisplay:TRUE];
-# endif /* !USE_IPHONE */
   }
 
 # ifdef USE_IPHONE
@@ -1498,9 +1308,10 @@ FAIL:
 - (void)showBlurb:(NSString *)text
 {
   CGRect frame = [self frame];
-  CGFloat rot;
+  // CGFloat rot;
   int size = ([self frame].size.width >= 768 ? 24 : 14);
   UIFont *font = [UIFont boldSystemFontOfSize: size];
+  // #### sizeWithFont deprecated as of iOS 7; use boundingRectWithSize.
   CGSize tsize = [text sizeWithFont:font
                   constrainedToSize:CGSizeMake(frame.size.width,
                                                frame.size.height)];
@@ -1513,38 +1324,8 @@ FAIL:
   frame = CGRectMake (0, 0, tsize.width, tsize.height);
   CGRect frame2 = frame;
 
-  UIInterfaceOrientation orient =
-    // Why are both of these wrong when starting up rotated??
-    [[UIDevice currentDevice] orientation];
-    // [rootViewController interfaceOrientation];
-
-  /* Get the text oriented properly, and move it to the bottom of the
-     screen.
-   */
-  switch (orient) {
-  case UIDeviceOrientationLandscapeRight:     
-    rot = -M_PI/2;
-    frame.origin.x = ([self frame].size.width
-                      - (tsize.width - tsize.height) / 2
-                      - tsize.height);
-    frame.origin.y = ([self frame].size.height - tsize.height) / 2;
-    break;
-  case UIDeviceOrientationLandscapeLeft:
-    rot = M_PI/2;
-    frame.origin.x = -(tsize.width - tsize.height) / 2;
-    frame.origin.y = ([self frame].size.height - tsize.height) / 2;
-    break;
-  case UIDeviceOrientationPortraitUpsideDown: 
-    rot = M_PI;
-    frame.origin.x = ([self frame].size.width  - tsize.width) / 2;
-    frame.origin.y = 0;
-    break;
-  default:
-    rot = 0;
-    frame.origin.x = ([self frame].size.width  - tsize.width) / 2;
-    frame.origin.y =  [self frame].size.height - tsize.height;
-    break;
-  }
+  frame.origin.x = ([self frame].size.width  - tsize.width) / 2;
+  frame.origin.y =  [self frame].size.height - tsize.height;
 
   if (aboutBox)
     [aboutBox removeFromSuperview];
@@ -1557,12 +1338,12 @@ FAIL:
    */
   aboutBox = [[UIView alloc] init];
   [aboutBox setFrame:frame];
-  aboutBox.transform = CGAffineTransformMakeRotation (rot);
+  // aboutBox.transform = CGAffineTransformMakeRotation (rot);
   
   UITextView *aboutText = [[UITextView alloc] initWithFrame:frame2];
  // aboutText.transform = CGAffineTransformMakeRotation (rot);
   aboutText.font = font;
-  aboutText.textAlignment = UITextAlignmentCenter;
+  aboutText.textAlignment = NSTextAlignmentCenter;
   aboutText.showsHorizontalScrollIndicator = NO;
   aboutText.showsVerticalScrollIndicator = NO;
   aboutText.scrollEnabled = NO;
